@@ -21,7 +21,6 @@
 /* #undef _GNU_SOURCE */
 /* #undef HAVE_ALLOCA_H */
 #define HAVE_ARPA_INET_H 1
-/* #undef HAVE_ASM_MSR_H */
 /* #undef HAVE_ASM_TERMBITS_H */
 /* #undef HAVE_CRYPT_H */
 #define HAVE_CURSES_H 1
@@ -97,7 +96,7 @@
 #define HAVE_TIME_H 1
 #define HAVE_UNISTD_H 1
 #define HAVE_UTIME_H 1
-/* #undef HAVE_VARARGS_H */
+#define HAVE_VARARGS_H 1
 /* #undef HAVE_SYS_UTIME_H */
 #define HAVE_SYS_WAIT_H 1
 #define HAVE_SYS_PARAM_H 1
@@ -184,10 +183,8 @@
 #define HAVE_PERROR 1
 #define HAVE_POLL 1
 /* #undef HAVE_POSIX_FALLOCATE */
-/* #undef HAVE_LINUX_FALLOC_H */
 /* #undef HAVE_FALLOC_PUNCH_HOLE_AND_KEEP_SIZE */
 #define HAVE_PREAD 1
-/* #undef HAVE_RDTSCLL */
 /* #undef HAVE_READ_REAL_TIME */
 /* #undef HAVE_PTHREAD_ATTR_CREATE */
 #define HAVE_PTHREAD_ATTR_GETGUARDSIZE 1
@@ -201,9 +198,9 @@
 #define HAVE_PTHREAD_SIGMASK 1
 /* #undef HAVE_PTHREAD_YIELD_NP */
 #define HAVE_PTHREAD_YIELD_ZERO_ARG 1
-/* #undef PTHREAD_ONCE_INITIALIZER */
+#define PTHREAD_ONCE_INITIALIZER PTHREAD_ONCE_INIT
 #define HAVE_PUTENV 1
-/* #undef HAVE_READDIR_R */
+#define HAVE_READDIR_R 1
 #define HAVE_READLINK 1
 #define HAVE_REALPATH 1
 #define HAVE_RENAME 1
@@ -217,6 +214,7 @@
 #define HAVE_SIGACTION 1
 /* #undef HAVE_SIGTHREADMASK */
 #define HAVE_SIGWAIT 1
+/* #undef HAVE_SIGWAITINFO */
 #define HAVE_SLEEP 1
 #define HAVE_SNPRINTF 1
 #define HAVE_STPCPY 1
@@ -335,7 +333,7 @@
     #define WORDS_BIGENDIAN 1
   #endif
 #else
-/* #undef WORDS_BIGENDIAN */
+#define WORDS_BIGENDIAN 1 
 #endif
 
 /* Define to `__inline__' or `__inline' if that's what the C compiler calls
@@ -378,10 +376,10 @@
 /* #undef HAVE_SVR3_SIGNALS */
 /* #undef HAVE_V7_SIGNALS */
 #define HAVE_ERR_remove_thread_state 1
+#define HAVE_X509_check_host 1
 
 /* #undef HAVE_SOLARIS_STYLE_GETHOST */
 
-#define HAVE_GCC_ATOMIC_BUILTINS 1
 #define HAVE_GCC_C11_ATOMICS 1
 /* #undef HAVE_SOLARIS_ATOMIC */
 /* #undef HAVE_DECL_SHM_HUGETLB */
@@ -399,8 +397,8 @@
 #define STACK_DIRECTION -1
 
 #define SYSTEM_TYPE "OpenBSD"
-#define MACHINE_TYPE "aarch64"
-#define DEFAULT_MACHINE "aarch64"
+#define MACHINE_TYPE "sparc64"
+#define DEFAULT_MACHINE "sparc64"
 /* #undef HAVE_DTRACE */
 
 #define SIGNAL_WITH_VIO_CLOSE 1
@@ -535,17 +533,17 @@
 /* #undef SO_EXT */
 
 #define MYSQL_VERSION_MAJOR 10
-#define MYSQL_VERSION_MINOR 3
-#define MYSQL_VERSION_PATCH 22
+#define MYSQL_VERSION_MINOR 4
+#define MYSQL_VERSION_PATCH 12
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.3.22"
+#define PACKAGE_STRING "MySQL Server 10.4.12"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.3.22"
-#define VERSION "10.3.22"
+#define PACKAGE_VERSION "10.4.12"
+#define VERSION "10.4.12"
 #define PROTOCOL_VERSION 10
 
 #define MALLOC_LIBRARY "system"
